% DWN_FUN_UTIL_plotting_FEV_CI.m plots the FEV with quantiles
% Input:
%  1. FEV_mat:              Structure containing FEV matrix
%                           eg. FEV_mat   = struct('A',fev_q); where
%                           fev_q is (h x N) matrix
%                           Note that the dimension will be checked such
%                           that N < h
%  2. var_list:             variable list, Number of row should be N
%  3. FLAG_scaling:         1. (FEV is scaled by 100)   0. not scaled
%  4. FLAG_FEV_subplotting: 1. customizing the subplot, 0. use the default
%                              setting of subplot
%  5. FEV_subplotting_DIM:  format of customized subplot (1 x 2) vector
%     eg.: FEV_subplotting_DIM = [4 2]
%  Note: If FEV_subplotting_DIM(1)*FEV_subplotting_DIM(2) < number of
%        variable, the default setting is used
%  6. FLAG_shading:   1. Shade the CI, 0. No shading 
%  7. tit:            Sup Title of graph (optional)
%  8. leg_tit:        legend name        (optional)
%
% It will generate a figure with N subplots of FEV with Confidence Interval

function DWN_FUN_UTIL_plotting_FEV_CI(FEV_mat, var_list, FLAG_scaling, FLAG_FEV_subplotting, FEV_subplotting_DIM, FLAG_shading, tit, leg_tit)

if nargin < 7
    tit = 'FEVs';
end

if nargin < 8
    if FLAG_shading == 1
        leg_tit = strvcat('Median','C.I.');
    else
        leg_tit = strvcat('Median','Lower Quantile','Upper Quantile');
    end
end

% Default Color Order in MATLAB for plotting
DefaultAxes_MATLAB = [...
         0         0    1.0000
         0    0.5000         0
    1.0000         0         0
         0    0.7500    0.7500
    0.7500         0    0.7500
    0.7500    0.7500         0
    0.2500    0.2500    0.2500];


name_mat = fieldnames(FEV_mat);
nmatrix = size(name_mat,1);

FEV_cell = cell(nmatrix,1);

for i = 1:nmatrix
    FEV_cell{i} = getfield(FEV_mat,strvcat(name_mat(i,:)));
    if FLAG_scaling == 1;
        FEV_cell{i} = 100*FEV_cell{i};
    end
    % ensuring that rows corresponds to time step and column
    % corresponds to variable
    [nr,nc] = size(FEV_cell{i});
    if nr < nc;
        FEV_cell{i} = FEV_cell{i}';
    end
end

for i = 1:nmatrix
    if (size(FEV_cell{1}) ~= size(FEV_cell{i}))
        disp(' ')
        disp(['Sizes of two FEV matrix are not consistent. '])
        disp(['Graph will not be plotted'])
        disp(' ')
        return
    end
end


t_step                  = size(FEV_cell{1},1);
number_of_plots_to_draw = size(FEV_cell{1},2);
n_var_list              = size(var_list,1);
% Check whether number_of_plots_to draw ~= n_var_list
if (n_var_list ~= number_of_plots_to_draw)
    disp(' ')
    disp(['Number of variable names is not consistent with number of FEV '...
        'considered'])
    disp(['Graph will not be plotted'])
    disp(' ')
    return
end


% Reset the DefaultAxesColorOrder for plot of impulse response
% Blue is the first color, % Second and third color is black
old_ColorOrder = DefaultAxes_MATLAB;
Black_Color = [0 0 0];
new_ColorOrder = [old_ColorOrder(1,:);Black_Color;Black_Color];

set(0,'DefaultAxesColorOrder',new_ColorOrder)

% Specify the Line Style Order
set(0,'DefaultAxesLineStyleOrder',{'-','->','-<'})

% Define Marker Type
marker_type = strvcat('o','*','s','d','p','h');

[nbplt,nr,nc,lr,lc,nstar] = DWN_FUN_UTIL_plotting_pltorg(number_of_plots_to_draw);

% Use customize one when FLAG_FEV_subplotting == 1
% Note that the customized one may not cover all the subplot when number of
% plot is smaller than FEV_subplotting_DIM(1)*FEV_subplotting_DIM(2), then
% use the default one
if FLAG_FEV_subplotting == 1 && number_of_plots_to_draw <= FEV_subplotting_DIM(1)*FEV_subplotting_DIM(2);
    [nbplt,nr,nc] = DWN_FUN_UTIL_plotting_pltorg_controlling_subplot(FEV_subplotting_DIM);
end
    

if nbplt == 1
    
    hh = figure('Name',[tit]);
    for j = 1:number_of_plots_to_draw
        mat_FEV = FEV_cell{1}(:,j);
        for k = 2:nmatrix
            mat_FEV = [mat_FEV FEV_cell{k}(:,j)];
        end
        
        subplot(nr,nc,j);
                        
        % newly added for shading part
        % new function 'DWN_FUN_UTIL_plotting_revr.m' is also needed and it reverses the rows of matrix
        if (nmatrix == 3) && (FLAG_shading == 1)
            grpyat    = [(0:t_step-1)' FEV_cell{2}(:,j); (t_step-1:-1:0)' flipud(FEV_cell{3}(:,j))];
            patch(grpyat(:,1),grpyat(:,2),[0.7 0.7 0.7],'edgecolor',[0.65 0.65 0.65]);
        end
        
        % for getting the maximum and minimum of y-axis
        hold on
        
        plot(0:t_step-1,mat_FEV,'linewidth',1);
        
        box on
        
        hold on

        % set the y-axis
        if FLAG_scaling == 1
            ylim([0 100]);
            
            set(gca,...
            'YTick',[0 20 40 60 80 100]);

        else
            ylim([0 1]);
            
            set(gca,...
            'YTick',[0 0.2 0.4 0.6 0.8 1]);

        end
        
        
        % Include or update the marker of each line      
        for i = 1:2
            set(findobj(gca,'Type','line','Color',new_ColorOrder(i,:),'Marker','none','LineStyle','-'),'Marker',marker_type(i,:))
        end
                       
        hold off

        xlim([0 t_step-1]);
        
        if FLAG_FEV_subplotting == 1;
            set(gca,'XTick',[0:10:t_step-1]);
        end
        
        title(deblank(var_list(j,:)),'Interpreter','none');
        if j == 1
            if FLAG_shading == 1    % with shading
                hold on
                legend('off');
                [legend_h,object_h,plot_h,text_strings] = legend('show');
                
                ind = [2,1];
                legend(plot_h(ind),leg_tit);
            else                    % without shading
                hold on
                legend('off');
                [legend_h,object_h,plot_h,text_strings] = legend('show');
                
                ind = [1,2,3];
                legend(plot_h(ind),leg_tit);                
            end
        end
        
    end

    % set the 'Super Title', The title in the Big Plot
    [ax,h3] = DWN_FUN_UTIL_plotting_suplabel(tit,'t');
    set(h3,'FontSize',14)
   
% This is the case when setting of Dynare subplot is used and more than 9
% variables are included in the system, then more than 1 figures are
% generated
else
    for fig = 1:nbplt-1
        hh = figure('Name',[tit]);
        for plt = 1:nstar
            mat_FEV = FEV_cell{1}(:,(fig-1)*nstar+plt);
            for k = 2:nmatrix
                mat_FEV = [mat_FEV FEV_cell{k}(:,(fig-1)*nstar+plt)];
            end
            subplot(nr,nc,plt);
            
            % newly added for shading part
            % new function 'DWN_FUN_UTIL_plotting_revr.m' is also needed and it reverses the rows of matrix
            if (nmatrix == 3) && (FLAG_shading == 1)
                grpyat    = [(0:t_step-1)' FEV_cell{2}(:,(fig-1)*nstar+plt); (t_step-1:-1:0)' flipud(FEV_cell{3}(:,(fig-1)*nstar+plt))];
                patch(grpyat(:,1),grpyat(:,2),[0.7 0.7 0.7],'edgecolor',[0.65 0.65 0.65]);
            end
            
            % for getting the maximum and minimum of y-axis
            hold on
            
            plot(0:t_step-1,mat_FEV,'linewidth',1);
            
            box on
            
            hold on

            % set the y-axis
            if FLAG_scaling == 1
                ylim([0 100]);
                
                set(gca,...
                    'YTick',[0 20 40 60 80 100]);
                
            else
                ylim([0 1]);
                
                set(gca,...
                    'YTick',[0 0.2 0.4 0.6 0.8 1]);
                
            end
            
            for i = 1:2
                set(findobj(gca,'Type','line','Color',new_ColorOrder(i,:),'Marker','none','LineStyle','-'),'Marker',marker_type(i,:))
            end
            

            %ax = ylim;
            hold off
            % xlim([1 t_step]);
            xlim([0 t_step-1]);
            
            if FLAG_FEV_subplotting == 1;
                set(gca,'XTick',[0:10:t_step-1]);
            end
            
            title(deblank(var_list((fig-1)*nstar+plt,:)), ...
                'Interpreter','none');
            if plt == 1
                if FLAG_shading == 1
                    hold on
                    legend('off');
                    [legend_h,object_h,plot_h,text_strings] = legend('show');
                    
                    ind = [2,1];
                    legend(plot_h(ind),leg_tit);
                else
                    hold on
                    legend('off');
                    [legend_h,object_h,plot_h,text_strings] = legend('show');
                    
                    ind = [1,2,3];
                    legend(plot_h(ind),leg_tit);
                end
            end
            
        end

        % set the 'Super Title', The title in the Big Plot
        [ax,h3] = DWN_FUN_UTIL_plotting_suplabel(tit,'t');
        set(h3,'FontSize',14)
    end
    
    m = 0;
    hh = figure('Name',[tit]);
    for plt = 1:number_of_plots_to_draw-(nbplt-1)*nstar;
        m = m+1;
        mat_FEV = FEV_cell{1}(:,(nbplt-1)*nstar+plt);
        for k = 2:nmatrix
            mat_FEV = [mat_FEV FEV_cell{k}(:,(nbplt-1)*nstar+plt)];
        end
        subplot(lr,lc,m);
        
        % newly added for shading part
        % new function 'DWN_FUN_UTIL_plotting_revr.m' is also needed and it reverses the rows
        % of matrix

        if (nmatrix == 3) && (FLAG_shading == 1)
            grpyat    = [(0:t_step-1)' FEV_cell{2}(:,(nbplt-1)*nstar+plt); (t_step-1:-1:0)' flipud(FEV_cell{3}(:,(nbplt-1)*nstar+plt))];
            patch(grpyat(:,1),grpyat(:,2),[0.7 0.7 0.7],'edgecolor',[0.65 0.65 0.65]);
        end

        % for getting the maximum and minimum of y-axis
        plot(0:t_step-1,mat_FEV,'linewidth',1);
        
        box on
        
        hold on

        % set the y-axis
        if FLAG_scaling == 1
            ylim([0 100]);
            
            set(gca,...
            'YTick',[0 20 40 60 80 100]);

        else
            ylim([0 1]);
            
            set(gca,...
            'YTick',[0 0.2 0.4 0.6 0.8 1]);

        end

        
        for i = 1:2
            set(findobj(gca,'Type','line','Color',new_ColorOrder(i,:),'Marker','none','LineStyle','-'),'Marker',marker_type(i,:))
        end        
       
        hold on
 
        plot([0 t_step-1],[0 0],'-r','linewidth',0.5);

        hold off
        xlim([0 t_step-1]);
        
        if FLAG_FEV_subplotting == 1;
            set(gca,'XTick',[0:10:t_step-1]);
        end
            
        title(deblank(var_list((nbplt-1)*nstar+plt,:)), ...
            'Interpreter','none');
        if plt == 1
            if FLAG_shading == 1
                hold on
                legend('off');
                [legend_h,object_h,plot_h,text_strings] = legend('show');
                
                ind = [2,1];
                legend(plot_h(ind),leg_tit);
            else
                hold on
                legend('off');
                [legend_h,object_h,plot_h,text_strings] = legend('show');
                
                ind = [1,2,3];
                legend(plot_h(ind),leg_tit);                
            end
        end
    end

    % set the 'Super Title', The title in the Big Plot
    [ax,h3] = DWN_FUN_UTIL_plotting_suplabel(tit,'t');
    set(h3,'FontSize',14)
end

% recover the orginal DefaultAxesColorOrder
set(0,'DefaultAxesColorOrder',old_ColorOrder)
