% DWN_FUN_plotting_IRF_sign_CI.m plots the IRF with CIs with indication of
% sign restriction period
% Input:
%  1. IRF_mat:              Structure matrix
%                           eg. IRF_mat   = struct('A',irf_q'); where
%                           IRF_MaxS_news_1 is (N x h) matrix
%  2. var_list:             variable list, Number of row should be N
%  3. FLAG_scaling:         1. (IRF is scaled by 100)   0. not scaled
%  4. FLAG_IRF_subplotting: 1. customizing the subplot, 0. use the default
%                              setting of subplot
%  5. IRF_subplotting_DIM:  format of customized subplot (1 x 2) vector
%     eg.: IRF_subplotting_DIM = [4 2]
%  Note: If IRF_subplotting_DIM(1)*IRF_subplotting_DIM(2) < number of
%        variable, the default setting is used
%  6. FLAG_shading:   1. Shade the CI, 0. No shading 
%  7. POS_HORIZON:    (1 x N) vector specifying the horizon for positive
%                     restriction
%  8. NEG_HORIZON:    (1 x N) vector specifying the horizon for negative
%                     restriction
%  9. HORIZON_ANT:    (1 x N) vector specifying the starting time step for
%                     sign restriction
%  10. tit:            Sup Title of graph
%  11. leg_tit:        legend name

function DWN_FUN_UTIL_plotting_SIGN_IRF_CI(IRF_mat, var_list, FLAG_scaling, FLAG_IRF_subplotting, IRF_subplotting_DIM, FLAG_shading, POS_HORIZON, NEG_HORIZON, HORIZON_ANT, tit, leg_tit)

if nargin < 10
    tit = 'IRFs';
end

if nargin < 11
    if FLAG_shading == 1
        leg_tit = strvcat('Median','C.I.');
    else
        leg_tit = strvcat('Median','Lower Quantile','Upper Quantile');
    end
end


name_mat = fieldnames(IRF_mat);
nmatrix = size(name_mat,1);

IRF_cell = cell(nmatrix,1);

for i = 1:nmatrix
    IRF_cell{i} = getfield(IRF_mat,strvcat(name_mat(i,:)));
    if FLAG_scaling == 1;
        IRF_cell{i} = 100*IRF_cell{i};
    end
    
    [nr,nc] = size(IRF_cell{i});
    if nr < nc;
        IRF_cell{i} = IRF_cell{i}';
    end
end

for i = 1:nmatrix
    if (size(IRF_cell{1}) ~= size(IRF_cell{i}))
        disp(' ')
        disp(['Sizes of two IRF matrix are not consistent. '])
        disp(['Graph will not be plotted'])
        disp(' ')
        return
    end
end


t_step                  = size(IRF_cell{1},1);
number_of_plots_to_draw = size(IRF_cell{1},2);
n_var_list              = size(var_list,1);
hor_i                   = [0:1:t_step-1];            % for indication of sign restriction period

% Check whether number_of_plots_to draw ~= n_var_list
if (n_var_list ~= number_of_plots_to_draw)
    disp(' ')
    disp(['Number of variable names is not consistent with number of IRF '...
        'considered'])
    disp(['Graph will not be plotted'])
    disp(' ')
    return
end

nleg = size(leg_tit,1);


% Reset the DefaultAxesColorOrder for plot of impulse response
% Blue is the first color, % Second and third color is black
old_ColorOrder = get(0,'DefaultAxesColorOrder');
Black_Color = [0 0 0];
new_ColorOrder = [old_ColorOrder(1,:);Black_Color;Black_Color];

set(0,'DefaultAxesColorOrder',new_ColorOrder)

% Specify the Line Style Order
set(0,'DefaultAxesLineStyleOrder',{'-','->','-<'})

% Define Marker Type
marker_type = strvcat('o','*','s','d','p','h');

% Use customize one when FLAG_FEV_subplotting == 1
% Note that the customized one may not cover all the subplot when number of
% plot is smaller than FEV_subplotting_DIM(1)*FEV_subplotting_DIM(2), then
% use the default one
[nbplt,nr,nc,lr,lc,nstar] = DWN_FUN_UTIL_plotting_pltorg(number_of_plots_to_draw);

if FLAG_IRF_subplotting == 1 && number_of_plots_to_draw <= IRF_subplotting_DIM(1)*IRF_subplotting_DIM(2);
    [nbplt,nr,nc] = DWN_FUN_UTIL_plotting_pltorg_controlling_subplot(IRF_subplotting_DIM);
end
    

if nbplt == 1
    
    hh = figure('Name',[tit]);
    for j = 1:number_of_plots_to_draw
        mat_IRF = IRF_cell{1}(:,j);
        for k = 2:nmatrix
            mat_IRF = [mat_IRF IRF_cell{k}(:,j)];
        end
        
        subplot(nr,nc,j);
        
        % Just want to get the default minimum and maximum y-axis
        plot(0:t_step-1,mat_IRF,'linewidth',1);
        yax = ylim;

        hold on
        
        cla;                                            % clear the plot
        
        % Shade the area for sign restriction region
        hold on
        if (POS_HORIZON(j) ~= 0)
            hndl_add =  area([HORIZON_ANT(j), HORIZON_ANT(j)+hor_i(POS_HORIZON(j))],[1.01 1.01]*(yax(1,1)-0.1),yax(1,2)+0.1,'FaceColor',[ 0.75 0.75 0.75 ]);
        end
        
        if (NEG_HORIZON(j) ~= 0)
            hndl_add =  area([HORIZON_ANT(j), HORIZON_ANT(j)+hor_i(NEG_HORIZON(j))],[1.01 1.01]*(yax(1,1)-0.1),yax(1,2)+0.1,'FaceColor',[ 0.75 0.75 0.75 ]);
        end

        hold on
                
        % newly added for shading part
        % new function 'DWN_FUN_plotting_revr.m' is also needed and it reverses the rows of matrix
        if ((nmatrix == 3) && (FLAG_shading == 1))
            grpyat    = [(0:t_step-1)' IRF_cell{2}(:,j); (t_step-1:-1:0)' flipud(IRF_cell{3}(:,j))];
            patch(grpyat(:,1),grpyat(:,2),[0.7 0.7 0.7],'edgecolor',[0.65 0.65 0.65]);
        end
        
        % for getting the maximum and minimum of y-axis
        hold on
        
        plot(0:t_step-1,mat_IRF,'linewidth',1);
        
        ylim([yax(1,1),yax(1,2)])
        
        hold on
        
        % add red line
        plot([0 t_step-1],[0 0],'-r','linewidth',0.5);
               
       
        box on
        
        hold on
        
        
        % Include or update the marker of each line      
        for i = 1:2
            set(findobj(gca,'Type','line','Color',new_ColorOrder(i,:),'Marker','none','LineStyle','-'),'Marker',marker_type(i,:))
        end
                       
        hold off
        % xlim([1 t_step]);
        xlim([0 t_step-1]);
        
        if FLAG_IRF_subplotting == 1;
            set(gca,'XTick',[0:10:t_step-1]);
        end
        
        title(deblank(var_list(j,:)),'Interpreter','none');
        if j == 1
            if FLAG_shading == 1
                hold on
                legend('off');
                [legend_h,object_h,plot_h,text_strings] = legend('show');
                
                if (POS_HORIZON(j) ~= 0) || (NEG_HORIZON(j) ~= 0)
                    ind = [3,2];
                else
                    ind = [2,1];
                end
                
                legend(plot_h(ind),leg_tit);
            else
                hold on
                legend('off');
                [legend_h,object_h,plot_h,text_strings] = legend('show');
                
                if (POS_HORIZON(j) ~= 0) || (NEG_HORIZON(j) ~= 0)
                    ind = [2,3,4];
                else
                    ind = [1,2,3];
                end

                legend(plot_h(ind),leg_tit);                
            end
        end
        
    end

    % set the 'Super Title', The title in the Big Plot
    [ax,h3] = DWN_FUN_UTIL_plotting_suplabel(tit,'t');
    set(h3,'FontSize',14)
   
% This is the case when setting of Dynare subplot is used and more than 9
% variables are included in the system, then more than 1 figures are
% generated
else
    for fig = 1:nbplt-1
        hh = figure('Name',[tit]);
        for plt = 1:nstar
            
            mat_IRF = IRF_cell{1}(:,(fig-1)*nstar+plt);
            for k = 2:nmatrix
                mat_IRF = [mat_IRF IRF_cell{k}(:,(fig-1)*nstar+plt)];
            end
            subplot(nr,nc,plt);
            
            % Just want to get the default minimum and maximum y-axis
            plot(0:t_step-1,mat_IRF,'linewidth',1);
            
            yax = ylim;
            
            ylim([yax(1,1),yax(1,2)])
            
            hold on
            
            cla;                                            % clear the plot
            
            % Shade the area for sign restriction region
            hold on
            
            if (POS_HORIZON((fig-1)*nstar+plt) ~= 0)
                hndl_add =  area([HORIZON_ANT((fig-1)*nstar+plt), HORIZON_ANT((fig-1)*nstar+plt)+hor_i(POS_HORIZON((fig-1)*nstar+plt))],[1.01 1.01]*(yax(1,1)-0.1),yax(1,2)+0.1,'FaceColor',[ 0.75 0.75 0.75 ]);
            end
            
            if (NEG_HORIZON((fig-1)*nstar+plt) ~= 0)
                hndl_add =  area([HORIZON_ANT((fig-1)*nstar+plt), HORIZON_ANT((fig-1)*nstar+plt)+hor_i(NEG_HORIZON((fig-1)*nstar+plt))],[1.01 1.01]*(yax(1,1)-0.1),yax(1,2)+0.1,'FaceColor',[ 0.75 0.75 0.75 ]);
            end            
            hold on

                        
            % newly added for shading part
            % new function 'DWN_FUN_plotting_revr.m' is also needed and it reverses the rows of matrix
            if (nmatrix == 3) && (FLAG_shading == 1)
                grpyat    = [(0:t_step-1)' IRF_cell{2}(:,(fig-1)*nstar+plt); (t_step-1:-1:0)' flipud(IRF_cell{3}(:,(fig-1)*nstar+plt))];
                patch(grpyat(:,1),grpyat(:,2),[0.7 0.7 0.7],'edgecolor',[0.65 0.65 0.65]);
            end
            
            % for getting the maximum and minimum of y-axis
            hold on
            
            plot(0:t_step-1,mat_IRF,'linewidth',1);
                        
            box on
            
            hold on
            
            plot([0 t_step-1],[0 0],'-r','linewidth',0.5);
            
            hold on                        

            
            for i = 1:2
                set(findobj(gca,'Type','line','Color',new_ColorOrder(i,:),'Marker','none','LineStyle','-'),'Marker',marker_type(i,:))
            end
            

            %ax = ylim;
            hold off
            % xlim([1 t_step]);
            xlim([0 t_step-1]);
            
            if FLAG_IRF_subplotting == 1;
                set(gca,'XTick',[0:10:t_step-1]);
            end
            
            title(deblank(var_list((fig-1)*nstar+plt,:)), ...
                'Interpreter','none');
            if plt == 1
                if FLAG_shading == 1
                    hold on
                    legend('off');
                    [legend_h,object_h,plot_h,text_strings] = legend('show');
                    
                    if (POS_HORIZON(j) ~= 0) || (NEG_HORIZON(j) ~= 0)
                        ind = [3,2];
                    else
                        ind = [2,1];
                    end
                    
                    legend(plot_h(ind),leg_tit);
                else
                    hold on
                    legend('off');
                    [legend_h,object_h,plot_h,text_strings] = legend('show');
                    
                    if (POS_HORIZON(j) ~= 0) || (NEG_HORIZON(j) ~= 0)
                        ind = [2,3,4];
                    else
                        ind = [1,2,3];
                    end
                
                    legend(plot_h(ind),leg_tit);
                end
            end
            
        end

        % set the 'Super Title', The title in the Big Plot
        [ax,h3] = DWN_FUN_UTIL_plotting_suplabel(tit,'t');
        set(h3,'FontSize',14)
    end
    
    m = 0;
    hh = figure('Name',[tit]);
    
    for plt = 1:number_of_plots_to_draw-(nbplt-1)*nstar;
        m = m+1;
        mat_IRF = IRF_cell{1}(:,(nbplt-1)*nstar+plt);
        for k = 2:nmatrix
            mat_IRF = [mat_IRF IRF_cell{k}(:,(nbplt-1)*nstar+plt)];
        end
        subplot(lr,lc,m);

        % Just want to get the default minimum and maximum y-axis
        plot(0:t_step-1,mat_IRF,'linewidth',1);
        
        yax = ylim;
        
        ylim([yax(1,1),yax(1,2)])
       
        hold on
        
        cla;                                            % clear the plot
        
        % Shade the area for sign restriction region
        hold on
        
        if (POS_HORIZON((nbplt-1)*nstar+plt) ~= 0)
            hndl_add =  area([HORIZON_ANT((nbplt-1)*nstar+plt), HORIZON_ANT((nbplt-1)*nstar+plt)+hor_i(POS_HORIZON((nbplt-1)*nstar+plt))],[1.01 1.01]*(yax(1,1)-0.1),yax(1,2)+0.1,'FaceColor',[ 0.75 0.75 0.75 ]);
        end
        
        if (NEG_HORIZON((nbplt-1)*nstar+plt) ~= 0)
            hndl_add =  area([HORIZON_ANT((nbplt-1)*nstar+plt), HORIZON_ANT((nbplt-1)*nstar+plt)+hor_i(NEG_HORIZON((nbplt-1)*nstar+plt))],[1.01 1.01]*(yax(1,1)-0.1),yax(1,2)+0.1,'FaceColor',[ 0.75 0.75 0.75 ]);
        end
        
        hold on
        
        if (nmatrix == 3) && (flag_shading == 1)
            grpyat    = [(0:t_step-1)' IRF_cell{2}(:,(nbplt-1)*nstar+plt); (t_step-1:-1:0)' flipud(IRF_cell{3}(:,(nbplt-1)*nstar+plt))];
            patch(grpyat(:,1),grpyat(:,2),[0.7 0.7 0.7],'edgecolor',[0.65 0.65 0.65]);
        end

        hold on
               
        % for getting the maximum and minimum of y-axis
        plot(0:t_step-1,mat_IRF,'linewidth',1);
        
        box on
        
        hold on
        
        plot([0 t_step-1],[0 0],'-r','linewidth',0.5);
        
        hold on        

        
        for i = 1:2
            set(findobj(gca,'Type','line','Color',new_ColorOrder(i,:),'Marker','none','LineStyle','-'),'Marker',marker_type(i,:))
        end        
       
        hold on
 
        plot([0 t_step-1],[0 0],'-r','linewidth',0.5);

        hold off
        xlim([0 t_step-1]);
        
        if FLAG_IRF_subplotting == 1;
            set(gca,'XTick',[0:10:t_step-1]);
        end
            
        title(deblank(var_list((nbplt-1)*nstar+plt,:)), ...
            'Interpreter','none');
        if plt == 1
            if FLAG_shading == 1
                hold on
                legend('off');
                [legend_h,object_h,plot_h,text_strings] = legend('show');
                
                if (POS_HORIZON(j) ~= 0) || (NEG_HORIZON(j) ~= 0)
                    ind = [3,2];
                else
                    ind = [2,1];
                end
                
                legend(plot_h(ind),leg_tit);
            else
                hold on
                legend('off');
                [legend_h,object_h,plot_h,text_strings] = legend('show');
                
                if (POS_HORIZON(j) ~= 0) || (NEG_HORIZON(j) ~= 0)
                    ind = [2,3,4];
                else
                    ind = [1,2,3];
                end
                
                legend(plot_h(ind),leg_tit);                
            end
        end
    end

    % set the 'Super Title', The title in the Big Plot
    [ax,h3] = DWN_FUN_UTIL_plotting_suplabel(tit,'t');
    set(h3,'FontSize',14)
end

% recover the orginal DefaultAxesColorOrder
set(0,'DefaultAxesColorOrder',old_ColorOrder)
