function IRF = IRF_horizons( x, nvar, nlag, npredetermined, horizons )
%
% Input:
% 
%   npredetermined  = nlag*nvar + 1, where 1 means inclusion of a constant term
% 
%   horizons        = the sequence of horizons, say, for horizon 0 to horizon 40, horizons = 0:40
%
%   The SVAR is as follows: 
%       y(t)'*A(0) = c' + y(t-1)'*A(1) + ... + y(t-p)'*A(p) + epsilon(t)'
%
%       y(t), epsilon(t), c - n x 1
%       A(i)                - n x n
%
%   A0    = A(0)
%   Aplus = [A(1); ... A(p); c']
%   x     = [vec(A0); vec(Aplus)]
%
% Output
%
%   The transpose of the IRF matrix at each horizon is first calculated:
%   R(0)        = inv(A(0))                                                         i = 0                   
%   R(i)        = ( R(0)*A(i) + R(1)*A(i-1) + ... + R(i-1)*A(1) )*R(0)              0 < i <= p
%   R(i)        = ( R(i-p)*A(p) + R(i-p+1)*A(p-1) + ... + R(i-1)*A(1) )*R(0)        p < i < inf
%   R(infinity) = inv( A(0) - A(1) - ... - A(p) )
%
%   IRF         = [ R(horizons(1))'; R(horizons(2))'; ...; R(horizons(k))' ],       where k = numel(horizons)
%               = [ L(horizons(1))'; L(horizons(2))'; ...; L(horizons(k))' ]
%

if numel(horizons) == 0
    IRF = zeros(0,nvar);
else
    
    maxh     = max(horizons(horizons ~= inf));
    long_run = (max(horizons) == inf);
    
    if maxh < nlag
        if long_run
            maxh = nlag;
        end
        q = maxh;
    else
        q = nlag;
    end
    
    % Construct A0 and Aplus from x
    n2      = nvar*nvar;
    
    A0      = reshape(x(1:n2),nvar,nvar);
    Aplus   = reshape(x(n2+1:n2+nvar*npredetermined),npredetermined,nvar);      % npredetermined = m = n*p+1
    A       = cell(nlag,1);
    
    % Each cell of R stores the transpose of each IRF matrix, starting from horizon 1
    R = cell(maxh,1);
    
    for i=1:q
        A{i} = Aplus((i-1)*nvar+1:i*nvar,:);
        X    = A0\A{i};
        for j=1:i-1
            X = X + R{j}*A{i-j};
        end
        R{i} = X/A0;    % R{i} = the transpose of the IRF matrix, L{i}
    end
    
    for i=nlag+1:maxh
        if nlag > 0
            X = R{i-nlag}*A{nlag};
        else
            X = zeros(nvar,nvar);
        end
        for j=1:nlag-1
            X = X + R{i-j}*A{j};
        end
        R{i} = X/A0;
    end
    
    if long_run
        R_long_run = A0;
        for j=1:nlag
            R_long_run=  R_long_run - A{j};
        end
        R_long_run = inv(R_long_run);   % the transpose of the long-run IRF matrix
    end
    
    % IRF stores the IRF matrices, [ IRF(0); IRF(1); ... ]
    IRF = zeros(numel(horizons)*nvar,nvar);
    for i=1:numel(horizons)
        
        if horizons(i) == inf
            IRF((i-1)*nvar+1:i*nvar,:) = R_long_run';
        elseif horizons(i) == 0
            IRF((i-1)*nvar+1:i*nvar,:) = inv(A0)';
        else
            IRF((i-1)*nvar+1:i*nvar,:) = R{horizons(i)}';
        end
        
    end
    
    
end




